package com.qmth.eds.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.util.Result;
import com.qmth.eds.common.util.ResultUtil;
import com.qmth.eds.service.ExamCourseMappingService;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/exam_course_mapping")
public class ExamCourseMappingController {

    @Resource
    private ExamCourseMappingService examCourseMappingService;


    /**
     * 查询
     *
     * @param semesterId             学期
     * @param examTypeId             考试类型
     * @param syncCourseCode         考务数据课程代码
     * @param cloudMarkingCourseCode 云阅卷课程代码
     * @param pageNumber             分页参数
     * @param pageSize               分页参数
     */
    @ApiOperation(value = "查询")
    @PostMapping("/page")
    public Result page(@RequestParam(value = "semesterId", required = false) Long semesterId,
                       @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                       @RequestParam(value = "syncCourseCode", required = false) String syncCourseCode,
                       @RequestParam(value = "cloudMarkingCourseCode", required = false) String cloudMarkingCourseCode,
                       @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(examCourseMappingService.pageData(semesterId, examTypeId, syncCourseCode, cloudMarkingCourseCode, pageNumber, pageSize));
    }

    /**
     * 导入对照关系文件
     *
     * @param semesterId 学期ID
     * @param examTypeId 考试类型ID
     * @param file       导入文件
     */
    @ApiOperation(value = "导入")
    @PostMapping("/import")
    public void page(@RequestParam("semesterId") Long semesterId,
                     @RequestParam("examTypeId") Long examTypeId,
                     @RequestParam("file") MultipartFile file) {
        examCourseMappingService.importData(semesterId, examTypeId, file);
    }

}
