package com.qmth.eds.common.tools;

import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.tools.signature.SignatureEntity;
import com.qmth.boot.tools.signature.SignatureType;
import com.qmth.eds.common.contant.SysSettingConstant;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.SysCollege;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.HttpKit;
import com.qmth.eds.core.config.DictionaryConfig;
import com.qmth.eds.service.CommonCacheService;
import com.qmth.eds.service.CommonService;
import com.qmth.eds.service.SysCollegeService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 云阅卷接口工具类
 */
@Service
public class CloudMarkingUtils {

    private static final String SUCCESS = "success";

    private static final String EXAMID = "examId";
    private static final String EXAM_NUMBER = "examNumber";
    private static final String STUDENT_CODE = "studentCode";
    private static final String COLLEGE = "college";
    private static final String CLASS_NAME = "className";
    private static final String TEACHER = "teacher";
    private static final String SUBJECT_CODE = "subjectCode";


    @Resource
    CommonService commonService;

    @Resource
    SysCollegeService sysCollegeService;

    /**
     * 考试成绩考生查询接口
     *
     * @param pageNo   分页参数
     * @param pageSize 分页参数
     */
    public List<JSONObject> getExam(Long schoolId, Long collegeId, int pageNo, int pageSize) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("pageNumber", String.valueOf(pageNo));
        paramMap.put("pageSize", String.valueOf(pageSize));

        String yyjRootUrl = commonService.getSysSettingValue(schoolId, SysSettingConstant.CLOUD_MARKING_ROOT_URL);
        String examUrl = SysSettingConstant.CLOUD_MARKING_EXAM_URL;

        if (StringUtils.isAnyBlank(yyjRootUrl, examUrl)) {
            throw ExceptionResultEnum.ERROR.exception("考试查询接口url未设置");
        }

        Map<String, String> header = createHeader(schoolId, examUrl, collegeId);
        //请求
        String examObject = HttpKit.sendPost(yyjRootUrl + examUrl, paramMap, header);
        return JSONObject.parseArray(JSONObject.toJSON(examObject).toString(), JSONObject.class);
    }

    /**
     * 考试成绩考生数量查询接口
     *
     * @param mapSource 参数
     * @param collegeId
     */
    public int getStudentCount(Long schoolId, Map<String, String> mapSource, Long collegeId) {
        Map<String, String> map = paramMap(mapSource);

        String yyjRootUrl = commonService.getSysSettingValue(schoolId, SysSettingConstant.CLOUD_MARKING_ROOT_URL);
        String studentCountUrl = SysSettingConstant.CLOUD_MARKING_STUDENT_COUNT_URL;

        if (StringUtils.isAnyBlank(yyjRootUrl, studentCountUrl)) {
            throw ExceptionResultEnum.ERROR.exception("考试成绩考生数量查询接口url未设置");
        }

        //header
        Map<String, String> headerMap = createHeader(schoolId, studentCountUrl, collegeId);
        //请求
        String res = HttpKit.sendPost(yyjRootUrl + studentCountUrl, map, headerMap);
        if (StringUtils.isNotBlank(res)) {
            JSONObject object = JSONObject.parseObject(res);
            return Integer.parseInt(object.get("totalCount").toString());
        }
        return 0;
    }

    /**
     * 考试成绩考生查询接口
     *
     * @param map       参数
     * @param collegeId
     * @param pageNo    分页参数
     * @param pageSize  分页参数
     */
    public List<JSONObject> getStudentScore(Long schoolId, Map<String, String> map, Long collegeId, int pageNo, int pageSize) {
        Map<String, String> paramMap = paramMap(map);

        paramMap.put("pageNumber", String.valueOf(pageNo));
        paramMap.put("pageSize", String.valueOf(pageSize));

        String yyjRootUrl = commonService.getSysSettingValue(schoolId, SysSettingConstant.CLOUD_MARKING_ROOT_URL);
        String studentScoreUrl = SysSettingConstant.CLOUD_MARKING_STUDENT_SCORE_URL;

        if (StringUtils.isAnyBlank(yyjRootUrl, studentScoreUrl)) {
            throw ExceptionResultEnum.ERROR.exception("考试成绩查询接口url未设置");
        }

        Map<String, String> header = createHeader(schoolId, studentScoreUrl, collegeId);
        //请求
        String studentObject = HttpKit.sendPost(yyjRootUrl + studentScoreUrl, paramMap, header);
        return JSONObject.parseArray(JSONObject.toJSON(studentObject).toString(), JSONObject.class);
    }

    /**
     * 云阅卷接口请求header
     *
     * @param url url
     */
    private Map<String, String> createHeader(Long schoolId, String url, Long collegeId) {
        SysCollege sysCollege = sysCollegeService.getById(collegeId);
        String accessKey = null;
        String accessSecret = null;
        if (sysCollege != null) {
            accessKey = sysCollege.getAccessKey();
            accessSecret = sysCollege.getAccessSecret();
        } else {
            accessKey = commonService.getSysSettingValue(schoolId, SysSettingConstant.CLOUD_MARKING_ACCESS_KEY);
            accessSecret = commonService.getSysSettingValue(schoolId, SysSettingConstant.CLOUD_MARKING_ACCESS_SECRET);
        }
        long time = System.currentTimeMillis();
        Map<String, String> headerMap = new HashMap<>();
        headerMap.put(SystemConstant.HEADER_TIME, String.valueOf(time));
        String signature = SignatureEntity.build(SignatureType.SECRET, "POST", url, time, accessKey, accessSecret);
        headerMap.put(SystemConstant.HEADER_AUTHORIZATION, signature);

        return headerMap;
    }

    private Map<String, String> paramMap(Map<String, String> mapSource) {
        Map<String, String> map = new HashMap<>();
        Integer examId = Integer.valueOf(mapSource.get(EXAMID));
        if (Objects.nonNull(examId)) {
            map.put(EXAMID, String.valueOf(examId));
        }
        String examCode = mapSource.get("examCode");
        if (StringUtils.isNotBlank(examCode)) {
            map.put("examCode", examCode);
        }
        String examNumber = mapSource.get(EXAM_NUMBER);
        if (StringUtils.isNotBlank(examNumber)) {
            map.put(EXAM_NUMBER, examNumber);
        }
        String studentCode = mapSource.get(STUDENT_CODE);
        if (StringUtils.isNotBlank(studentCode)) {
            map.put(STUDENT_CODE, studentCode);
        }
        String subjectCode = mapSource.get(SUBJECT_CODE);
        if (StringUtils.isNotBlank(subjectCode)) {
            map.put(SUBJECT_CODE, subjectCode);
        }
        String college = mapSource.get(COLLEGE);
        if (StringUtils.isNotBlank(college)) {
            map.put(COLLEGE, college);
        }
        String className = mapSource.get(CLASS_NAME);
        if (StringUtils.isNotBlank(className)) {
            map.put(CLASS_NAME, className);
        }
        String teacher = mapSource.get(TEACHER);
        if (StringUtils.isNotBlank(teacher)) {
            map.put(TEACHER, teacher);
        }
        return map;
    }
}
