package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.common.entity.ExamCourseMapping;
import com.qmth.eds.common.entity.ExamSyncStudent;
import com.qmth.eds.common.entity.ExamSyncTotal;
import com.qmth.eds.mapper.ExamSyncStudentMapper;
import com.qmth.eds.service.ExamCourseMappingService;
import com.qmth.eds.service.ExamSyncStudentService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;
import java.util.Set;

@Service
public class ExamSyncStudentServiceImpl extends ServiceImpl<ExamSyncStudentMapper, ExamSyncStudent> implements ExamSyncStudentService {

    @Resource
    private ExamCourseMappingService examCourseMappingService;

    @Resource
    ExamSyncStudentMapper examSyncStudentMapper;

    @Override
    public List<ExamSyncStudent> listByExamSyncTotalId(ExamSyncTotal examSyncTotal) {
        Long examSyncTotalId = examSyncTotal.getId();
        Long schoolId = examSyncTotal.getSchoolId();
        Long semesterId = examSyncTotal.getSemesterId();
        Long examTypeId = examSyncTotal.getExamTypeId();

        QueryWrapper<ExamSyncStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamSyncStudent::getExamSyncTotalId, examSyncTotalId);
        List<ExamSyncStudent> examSyncStudents = this.list(queryWrapper);

        QueryWrapper<ExamCourseMapping> examCourseMappingQueryWrapper = new QueryWrapper<>();
        examCourseMappingQueryWrapper.lambda().eq(ExamCourseMapping::getSchoolId, schoolId)
                .eq(ExamCourseMapping::getSemesterId, semesterId)
                .eq(ExamCourseMapping::getExamTypeId, examTypeId);
        List<ExamCourseMapping> examCourseMappings = examCourseMappingService.list(examCourseMappingQueryWrapper);
        if (!examCourseMappings.isEmpty()) {
            for (ExamSyncStudent examSyncStudent : examSyncStudents) {
                Optional<ExamCourseMapping> examCourseMapping = examCourseMappings.stream().filter(m -> m.getSyncCourseCode().equals(examSyncStudent.getKch())).findFirst();
                if (examCourseMapping.isPresent()) {
                    examSyncStudent.setCloudMarkingCourseCode(examCourseMapping.get().getCloudMarkingCourseCode());
                }
            }
        }

        return examSyncStudents;
    }

    /**
     * 根据机构id、考试id、科目代码查询考查学院
     *
     * @param schoolId
     * @param orgId
     * @param examId
     * @param courseCode
     * @param cloudMarkSubjectCodeSet
     * @return
     */
    @Override
    public List<ExamSyncStudent> queryByExamineCollegeList(Long schoolId, Long orgId, Long examId, String courseCode, Set<String> cloudMarkSubjectCodeSet) {
        return examSyncStudentMapper.queryByExamineCollegeList(schoolId, orgId, examId, courseCode, cloudMarkSubjectCodeSet);
    }
}
